# -*- coding: utf-8 -*-
# Daum Movie

import urllib, unicodedata, os

DAUM_MOVIE_SRCH   = "http://movie.daum.net/data/movie/search/v2/movie.json?size=20&start=1&searchText=%s"

DAUM_MOVIE_DETAIL = "http://movie.daum.net/moviedb/main?movieId=%s"
# DAUM_MOVIE_DETAIL = "http://movie.daum.net/data/movie/movie_info/detail.json?movieId=%s"
DAUM_MOVIE_CAST   = "http://movie.daum.net/data/movie/movie_info/cast_crew.json?pageNo=1&pageSize=100&movieId=%s"
DAUM_MOVIE_PHOTO  = "http://movie.daum.net/data/movie/photo/movie/list.json?pageNo=1&pageSize=100&id=%s"

DAUM_TV_SRCH      = "https://search.daum.net/search?w=tot&q=%s"
DAUM_TV_DETAIL    = "https://search.daum.net/search?w=tv&q=%s&irk=%s&irt=tv-program&DA=TVP"

IMDB_TITLE_SRCH   = "http://www.google.com/search?q=site:imdb.com+%s"
TVDB_TITLE_SRCH   = "http://thetvdb.com/api/GetSeries.php?seriesname=%s"

RE_YEAR_IN_NAME   =  Regex('\((\d+)\)')
RE_MOVIE_ID       =  Regex("movieId=(\d+)")
RE_TV_ID          =  Regex("tvProgramId=(\d+)")
RE_PHOTO_SIZE     =  Regex("/C\d+x\d+/")
RE_IMDB_ID        =  Regex("/(tt\d+)/")

JSON_MAX_SIZE     = 10 * 1024 * 1024

DAUM_CR_TO_MPAA_CR = {
    u'전체관람가': {
        'KMRB': 'kr/A',
        'MPAA': 'G'
    },
    u'12세이상관람가': {
        'KMRB': 'kr/12',
        'MPAA': 'PG'
    },
    u'15세이상관람가': {
        'KMRB': 'kr/15',
        'MPAA': 'PG-13'
    },
    u'청소년관람불가': {
        'KMRB': 'kr/R',
        'MPAA': 'R'
    },
    u'제한상영가': {     # 어느 여름날 밤에 (2016)
        'KMRB': 'kr/X',
        'MPAA': 'NC-17'
    }
}

def Start():
  HTTP.CacheTime = CACHE_1HOUR * 12
  HTTP.Headers['Accept'] = 'text/html, application/json'

####################################################################################################
def searchDaumMovie(results, media, lang):
  media_name = media.name
  media_name = unicodedata.normalize('NFKC', unicode(media_name)).strip()
  Log.Debug("search: %s %s" %(media_name, media.year))

  data = JSON.ObjectFromURL(url=DAUM_MOVIE_SRCH % (urllib.quote(media_name.encode('utf8'))))
  items = data['data']
  for item in items:
    year = str(item['prodYear'])
    title = String.DecodeHTMLEntities(String.StripTags(item['titleKo'])).strip()
    id = str(item['movieId'])
    if year == media.year:
      score = 95
    elif len(items) == 1:
      score = 80
    else:
      score = 10
    Log.Debug('ID=%s, media_name=%s, title=%s, year=%s, score=%d' %(id, media_name, title, year, score))
    results.Append(MetadataSearchResult(id=id, name=title, year=year, score=score, lang=lang))

def searchDaumTV(results, media, lang):
  media_name = unicodedata.normalize('NFKC', unicode(media.show)).strip()
  media_year = media.year
  # if not media_year and media.filename:
  #   match = Regex('\D(\d{2})[01]\d[0-3]\d\D').search(os.path.basename(urllib.unquote(media.filename)))
  #   if match:
  #     media_year = '20' + match.group(1)
  Log.Debug("search: %s %s" %(media_name, media_year))

  # TV검색
  html = HTML.ElementFromURL(DAUM_TV_SRCH % urllib.quote(media_name.encode('utf8')))
  try:
    tvp = html.xpath('//div[@id="tvpColl"]')[0]
  except:
    Log.Debug('No TV matches found')
    return

  items = []
  id = Regex('irk=(\d+)').search(tvp.xpath('//a[@class="tit_info"]/@href')[-1]).group(1)
  title = ''.join(tvp.xpath('//a[@class="tit_info"]/text()'))
  try:
    year = Regex('(\d{4})\.\d+\.\d+~').search(tvp.xpath('//div[@class="head_cont"]//span[@class="txt_summary"][last()]')[0].text).group(1)
  except: year = None
  items.append({ 'id': id, 'title': title, 'year': year })

  # TV검색 > 시리즈
  more_a = tvp.xpath(u'//a[span[.="시리즈 더보기"]]')
  if more_a:
    html = HTML.ElementFromURL('https://search.daum.net/search%s' % more_a[0].get('href'))
    for li in html.xpath('//div[@id="series"]//li'):
      a = li.xpath('.//a')[1]
      id = Regex('irk=(\d+)').search(a.get('href')).group(1)
      title = a.text
      try:
        year = Regex('(\d{4})\.\d+').search(li.xpath('./span')[0].text).group(1)
        items.append({ 'id': id, 'title': title, 'year': year })
      except: pass
  else:
    lis = tvp.xpath('//div[@id="tv_series"]//li')
    for li in lis:
      id = Regex('irk=(\d+)').search(li.xpath('./a/@href')[0]).group(1)
      title = li.xpath('./a')[0].text
      try:
        year = Regex('(\d{4})\.\d+').search(li.xpath('./span')[0].text).group(1)
        items.append({ 'id': id, 'title': title, 'year': year })
      except: pass

  # TV검색 > 동명 콘텐츠
  spans = tvp.xpath(u'//div[contains(@class,"coll_etc")]//span[.="(동명프로그램)"]')
  for span in spans:
    year = Regex('(\d{4})').search(span.xpath('./preceding-sibling::span[1]')[0].text).group(1)
    a = span.xpath('./preceding-sibling::a[1]')[0]
    id = Regex('irk=(\d+)').search(a.get('href')).group(1)
    title = a.text.strip()
    items.append({ 'id': id, 'title': title, 'year': year })

  for idx, item in enumerate(items):
    score = 80 - idx * 20
    if media_year:
      score += (2 - min(2, abs(int(media_year) - int(item['year'])))) * 5
    if score < 10:
      score = 10
    Log.Debug('ID=%s, media_name=%s, title=%s, year=%s, score=%d' %(item['id'], media_name, item['title'], item['year'], score))
    results.Append(MetadataSearchResult(id=item['id'], name=item['title'], year=item['year'], score=score, lang=lang))

def updateDaumMovie(metadata):
  # (1) from detail page
  poster_url = None

  try:
    html = HTML.ElementFromURL(DAUM_MOVIE_DETAIL % metadata.id)
    title = html.xpath('//div[@class="subject_movie"]/strong')[0].text
    match = Regex('(.*?) \((\d{4})\)').search(title)
    metadata.title = match.group(1)
    metadata.title_sort = unicodedata.normalize('NFKD' if Prefs['use_title_decomposition'] else 'NFKC', metadata.title)
    metadata.year = int(match.group(2))
    metadata.original_title = html.xpath('//span[@class="txt_movie"]')[0].text
    metadata.rating = float(html.xpath('//em[@class="emph_grade"]')[0].text)
    # 장르
    metadata.genres.clear()
    dds = html.xpath('//dl[contains(@class, "list_movie")]/dd')
    for genre in dds.pop(0).text.split('/'):
        metadata.genres.add(genre)
    # 나라
    metadata.countries.clear()
    for country in dds.pop(0).text.split(','):
        metadata.countries.add(country.strip())
    # 개봉일 (optional)
    match = Regex(u'(\d{4}\.\d{2}\.\d{2})\s*개봉').search(dds[0].text)
    if match:
      metadata.originally_available_at = Datetime.ParseDate(match.group(1)).date()
      dds.pop(0)
    # 재개봉 (optional)
    match = Regex(u'(\d{4}\.\d{2}\.\d{2})\s*\(재개봉\)').search(dds[0].text)
    if match:
      dds.pop(0)
    # 상영시간, 등급 (optional)
    match = Regex(u'(\d+)분(?:, (.*?)\s*$)?').search(dds.pop(0).text)
    if match:
      metadata.duration = int(match.group(1))
      cr = match.group(2)
      if cr:
        match = Regex(u'미국 (.*) 등급').search(cr)
        if match:
          metadata.content_rating = match.group(1)
        elif cr in DAUM_CR_TO_MPAA_CR:
          metadata.content_rating = DAUM_CR_TO_MPAA_CR[cr]['MPAA' if Prefs['use_mpaa'] else 'KMRB']
        else:
          metadata.content_rating = 'kr/' + cr
    # Log.Debug('genre=%s, country=%s' %(','.join(g for g in metadata.genres), ','.join(c for c in metadata.countries)))
    # Log.Debug('oaa=%s, duration=%s, content_rating=%s' %(metadata.originally_available_at, metadata.duration, metadata.content_rating))
    metadata.summary = '\n'.join(txt.strip() for txt in html.xpath('//div[@class="desc_movie"]/p//text()'))
    poster_url = html.xpath('//img[@class="img_summary"]/@src')[0]
  except Exception, e:
    Log.Debug(repr(e))
    pass

  # (2) cast crew
  directors = list()
  producers = list()
  writers = list()
  roles = list()

  data = JSON.ObjectFromURL(url=DAUM_MOVIE_CAST % metadata.id)
  for item in data['data']:
    cast = item['castcrew']
    if cast['castcrewCastName'] in [u'감독', u'연출']:
      director = dict()
      director['name'] = item['nameKo'] if item['nameKo'] else item['nameEn']
      if item['photo']['fullname']:
        director['photo'] = item['photo']['fullname']
      directors.append(director)
    elif cast['castcrewCastName'] == u'제작':
      producer = dict()
      producer['name'] = item['nameKo'] if item['nameKo'] else item['nameEn']
      if item['photo']['fullname']:
        producer['photo'] = item['photo']['fullname']
      producers.append(producer)
    elif cast['castcrewCastName'] in [u'극본', u'각본']:
      writer = dict()
      writer['name'] = item['nameKo'] if item['nameKo'] else item['nameEn']
      if item['photo']['fullname']:
        writer['photo'] = item['photo']['fullname']
      writers.append(writer)
    elif cast['castcrewCastName'] in [u'주연', u'조연', u'출연', u'진행']:
      role = dict()
      role['role'] = cast['castcrewTitleKo']
      role['name'] = item['nameKo'] if item['nameKo'] else item['nameEn']
      if item['photo']['fullname']:
        role['photo'] = item['photo']['fullname']
      roles.append(role)
    # else:
    #   Log.Debug("unknown role: castcrewCastName=%s" % cast['castcrewCastName'])

  if directors:
    metadata.directors.clear()
    for director in directors:
      meta_director = metadata.directors.new()
      if 'name' in director:
        meta_director.name = director['name']
      if 'photo' in director:
        meta_director.photo = director['photo']
  if producers:
    metadata.producers.clear()
    for producer in producers:
      meta_producer = metadata.producers.new()
      if 'name' in producer:
        meta_producer.name = producer['name']
      if 'photo' in producer:
        meta_producer.photo = producer['photo']
  if writers:
    metadata.writers.clear()
    for writer in writers:
      meta_writer = metadata.writers.new()
      if 'name' in writer:
        meta_writer.name = writer['name']
      if 'photo' in writer:
        meta_writer.photo = writer['photo']
  if roles:
    metadata.roles.clear()
    for role in roles:
      meta_role = metadata.roles.new()
      if 'role' in role:
        meta_role.role = role['role']
      if 'name' in role:
        meta_role.name = role['name']
      if 'photo' in role:
        meta_role.photo = role['photo']

  # (3) from photo page
  url_tmpl = DAUM_MOVIE_PHOTO
  data = JSON.ObjectFromURL(url=url_tmpl % metadata.id)
  max_poster = int(Prefs['max_num_posters'])
  max_art = int(Prefs['max_num_arts'])
  idx_poster = 0
  idx_art = 0
  for item in data['data']:
    if item['photoCategory'] == '1' and idx_poster < max_poster:
      art_url = item['fullname']
      if not art_url: continue
      #art_url = RE_PHOTO_SIZE.sub("/image/", art_url)
      idx_poster += 1
      try: metadata.posters[art_url] = Proxy.Preview(HTTP.Request(item['thumbnail']), sort_order = idx_poster)
      #try: metadata.posters[art_url] = Proxy.Preview(HTTP.Request(item['thumbnail'], None, { 'Referer': url_tmpl }), sort_order = idx_poster)
      except: pass
    elif item['photoCategory'] in ['2', '50'] and idx_art < max_art:
      art_url = item['fullname']
      if not art_url: continue
      #art_url = RE_PHOTO_SIZE.sub("/image/", art_url)
      idx_art += 1
      try: metadata.art[art_url] = Proxy.Preview(HTTP.Request(item['thumbnail']), sort_order = idx_art)
      #try: metadata.art[art_url] = Proxy.Preview(HTTP.Request(item['thumbnail'], None, { 'Referer': url_tmpl }), sort_order = idx_art)
      except: pass
  Log.Debug('Total %d posters, %d artworks' %(idx_poster, idx_art))
  if idx_poster == 0:
    if poster_url:
      poster = HTTP.Request( poster_url )
      try: metadata.posters[poster_url] = Proxy.Media(poster)
      except: pass
    # else:
    #   url = 'http://m.movie.daum.net/m/tv/main?tvProgramId=%s' % metadata.id
    #   html = HTML.ElementFromURL( url )
    #   arts = html.xpath('//img[@class="thumb_program"]')
    #   for art in arts:
    #     art_url = art.attrib['src']
    #     if not art_url: continue
    #     art = HTTP.Request( art_url )
    #     idx_poster += 1
    #     metadata.posters[art_url] = Proxy.Preview(art, sort_order = idx_poster)

    # (5) fill missing info
    # if Prefs['override_tv_id'] != 'None':
    #   page = HTTP.Request(DAUM_TV_DETAIL2 % metadata.id).content
    #   match = Regex('<em class="title_AKA"> *<span class="eng">([^<]*)</span>').search(page)
    #   if match:
    #     metadata.original_title = match.group(1).strip()

def updateDaumTV(metadata, media):
  # (1) from detail page
  try:
    html = HTML.ElementFromURL(DAUM_TV_DETAIL % (urllib.quote(media.title.encode('utf8')), metadata.id))
    metadata.title = html.xpath('//div[@class="tit_program"]/strong')[0].text
    metadata.title_sort = unicodedata.normalize('NFKD' if Prefs['use_title_decomposition'] else 'NFKC', metadata.title)
    metadata.original_title = ''
    metadata.rating = None
    metadata.genres.clear()
    # 드라마 (24부작)
    metadata.genres.add(Regex(u'(.*?)(?:\u00A0(\(.*\)))?$').search(html.xpath(u'//dt[.="장르"]/following-sibling::dd/text()')[0]).group(1))
    metadata.studio = html.xpath('//div[@class="txt_summary"]/span[1]')[0].text
    match = Regex('(\d+\.\d+\.\d+)~(\d+\.\d+\.\d+)?').search(html.xpath('//div[@class="txt_summary"]/span[last()]')[0].text or '')
    if match:
      metadata.originally_available_at = Datetime.ParseDate(match.group(1)).date()
    metadata.summary = String.DecodeHTMLEntities(String.StripTags(html.xpath(u'//dt[.="소개"]/following-sibling::dd')[0].text).strip())

    # //search1.kakaocdn.net/thumb/C232x336.q85/?fname=http%3A%2F%2Ft1.daumcdn.net%2Fcontentshub%2Fsdb%2Ff63c5467710f5669caac131943855dfea31011003e57e674832fe8b16b946aa8
    # poster_url = urlparse.parse_qs(urlparse.urlparse(html.xpath('//div[@class="info_cont"]/div[@class="wrap_thumb"]/a/img/@src')[0]).query)['fname'][0]
    poster_url = urllib.unquote(Regex('fname=(.*)').search(html.xpath('//div[@class="info_cont"]/div[@class="wrap_thumb"]/a/img/@src')[0]).group(1))
    metadata.posters[poster_url] = Proxy.Media(HTTP.Request(poster_url))
  except Exception, e:
    Log.Debug(repr(e))
    pass

  # (2) cast crew
  directors = list()
  producers = list()
  writers = list()
  roles = list()

  for item in html.xpath('//div[@class="wrap_col lst"]/ul/li'):
    try:
      role = item.xpath('./span[@class="sub_name"]/text()')[0].strip().replace(u'이전 ', '')
      cast = dict()
      cast['name'] = item.xpath('./span[@class="txt_name"]/a/text()')[0]
      cast['photo'] = item.xpath('./div/a/img/@src')[0]
      if role in [u'감독', u'연출', u'조감독']:
        directors.append(cast)
      elif role in [u'제작', u'프로듀서', u'책임프로듀서', u'기획']:
        producers.append(cast)
      elif role in [u'극본', u'각본', u'원작']:
        writers.append(cast)
      else:
        Log('Unknown role %s' % role)
    except: pass

  for item in html.xpath('//div[@class="wrap_col castingList"]/ul/li'):
    try:
      cast = dict()
      a = item.xpath('./span[@class="sub_name"]/a')
      if a:
        cast['name'] = a[0].text
        cast['role'] = item.xpath('./span[@class="txt_name"]/a')[0].text
        cast['photo'] = 'https:' + item.xpath('./div/a/img/@src')[0]
      else:
        cast['name'] = item.xpath('./span[@class="txt_name"]/a')[0].text
        cast['role'] = item.xpath('./span[@class="sub_name"]')[0].text.strip()
        cast['photo'] = 'https:' + item.xpath('./div/a/img/@src')[0]
      roles.append(cast)
    except: pass

  if roles:
    metadata.roles.clear()
    for role in roles:
      meta_role = metadata.roles.new()
      if 'role' in role:
        meta_role.role = role['role']
      if 'name' in role:
        meta_role.name = role['name']
      if 'photo' in role:
        meta_role.photo = role['photo']

  # (4) from episode page
  for a in html.xpath('//ul[@id="clipDateList"]/li/a'):
    season_num = '1'
    episode_num = a.xpath(u'substring-before(./span[@class="txt_episode"],"회")')
    try:
      episode_date = Datetime.ParseDate(a.xpath('./parent::li/@data-clip')[0], '%Y%m%d').date()
    except: continue
    if not episode_num: continue    # 시청지도서
    date_based_season_num = episode_date.year
    date_based_episode_num = episode_date.strftime('%Y-%m-%d')
    if ((season_num in media.seasons and episode_num in media.seasons[season_num].episodes) or
        (date_based_season_num in media.seasons and date_based_episode_num in media.seasons[date_based_season_num].episodes)):
      page = HTML.ElementFromURL('https://search.daum.net/search' + a.get('href'))
      episode = metadata.seasons[season_num].episodes[episode_num]
      subtitle = page.xpath('//p[@class="episode_desc"]/strong/text()')
      episode.summary = '\n'.join(txt.strip() for txt in page.xpath('//p[@class="episode_desc"]/text()')).strip()
      episode.originally_available_at = episode_date
      episode.title = subtitle[0] if subtitle else date_based_episode_num
      episode.rating = None

      if directors:
        episode.directors.clear()
        for director in directors:
          meta_director = episode.directors.new()
          if 'name' in director:
            meta_director.name = director['name']
          if 'photo' in director:
            meta_director.photo = director['photo']
      if writers:
        episode.writers.clear()
        for writer in writers:
          meta_writer = episode.writers.new()
          if 'name' in writer:
            meta_writer.name = writer['name']
          if 'photo' in writer:
            meta_writer.photo = writer['photo']

  # TV검색 > TV정보 > 공식홈
  home = html.xpath(u'//a[span[contains(.,"공식홈")]]/@href')
  if home:
    if 'www.imbc.com' in home[0]:
      page = HTML.ElementFromURL(home[0])
      for prv in page.xpath('//div[@class="roll-ban-event"]/ul/li/img/@src'):
        if prv not in metadata.art:
          try: metadata.art[prv] = Proxy.Preview(HTTP.Request(prv), sort_order = len(metadata.art) + 1)
          except Exception, e: Log(str(e))

  # TV검색 > TV정보 > 다시보기
  vod = html.xpath(u'//a[span[contains(.,"다시보기")]]/@href')
  if vod:
    replay_url = vod[0]
  else:
    if home and 'program.kbs.co.kr' in home[0]:
      # 카카오VOD > http://program.kbs.co.kr/2tv/drama/dramaspecial2018/pc/list.html?smenu=c2cc5a
      replay_url = home[0] + 'list.html?smenu=c2cc5a'
    else:
      replay_url = None
      if Regex('MBC|SBS|KBS').match(metadata.studio):
        Log.Debug('No replay URL for [%s] %s' % (metadata.studio, metadata.title))

  if replay_url:
    if 'www.imbc.com' in replay_url:
      try:
        prog_codes = []
        # http://www.imbc.com/broad/tv/ent/challenge/vod/index.html
        page = HTML.ElementFromURL(replay_url)
        prog_codes.append(Regex('var progCode = "(.*?)";').search(page.xpath('//script[contains(.,"var progCode = ")]/text()')[0]).group(1))
        # for season_vod in page.xpath('//map[@name="vod"]/area/@href'):
        #   # http://www.imbc.com/broad/tv/ent/challenge/vod1/
        #   # http://www.imbc.com/broad/tv/ent/challenge/vod2/
        #   page = HTML.ElementFromURL(season_vod)
        #   prog_codes.append(Regex('var progCode = "(.*?)";').search(page.xpath('//script[contains(.,"var progCode = ")]/text()')[0]).group(1))
        for prog_code in prog_codes:
          page = HTTP.Request('http://vodmall.imbc.com/util/wwwUtil_sbox.aspx?kind=image&progCode=%s' % prog_code).content
          years = Regex("<option value='(\d+)'>").findall(page)
          for year in years:
            page = unicode(HTTP.Request('http://vodmall.imbc.com/util/wwwUtil_sbox_contents.aspx?progCode=%s&yyyy=%s&callback=jQuery1123011760857070017172_1538059867383&_=1538059867389'
                % (prog_code, year)).content, 'euc-kr')
            bcasts = JSON.ObjectFromString(Regex('jQuery1123011760857070017172_1538059867383\((.*)\)$').search(page).group(1))
            for bcast in bcasts:
              # if u'특집' in bcast['ContentNumber'] or u'스페셜' in bcast['ContentNumber']:   # 특집05회, 특집회, 추석특집회, 스페셜회
              #   # Log('ignoring %s' % bcast['ContentNumber'])
              #   continue
              season_num = '1'
              episode_date = Datetime.ParseDate(bcast['BroadDate'], '%Y-%m-%d').date()
              match = Regex(u'^(\d+(-\d+)?)회$').search(bcast['ContentNumber'])  # 7-8회, 1회
              if match:
                for episode_num in match.group(1).split('-'):
                  date_based_season_num = episode_date.year
                  date_based_episode_num = episode_date.strftime('%Y-%m-%d')
                  if ((season_num in media.seasons and episode_num in media.seasons[season_num].episodes) or
                      (date_based_season_num in media.seasons and date_based_episode_num in media.seasons[date_based_season_num].episodes)):
                    episode = metadata.seasons[season_num].episodes[episode_num]
                    if episode.summary and u'회차정보가 없습니다' not in episode.summary:
                      continue
                    page = unicode(HTTP.Request('http://vodmall.imbc.com/util/wwwUtil_json.aspx?kind=image&progCode=%s&callback=jQuery111104041909438012061_1538031601249&_=1538031601252'
                        % bcast['BroadCastID']).content, 'euc-kr')
                    info = JSON.ObjectFromString(Regex('jQuery111104041909438012061_1538031601249\((.*)\)$').search(page).group(1))[0]
                    episode.summary = info['Content'].replace('\r\n', '\n').replace('<br><br>', '\n').replace('<br>', '').strip()
                    episode.originally_available_at = episode_date
                    episode.title = info['Title']
                    episode.rating = None
      except Exception, e:
        Log.Debug(repr(e))
        pass

    elif 'programs.sbs.co.kr' in replay_url:
      try:
        # http://programs.sbs.co.kr/enter/jungle/vods/50479
        programcd, mnuid = Regex('programs\.sbs\.co\.kr/(.+?)/(.+?)/vods/(.+)$').search(replay_url).group(2, 3)

        # http://static.apis.sbs.co.kr/program-api/1.0/menu/jungle
        menu = JSON.ObjectFromURL('http://static.apis.sbs.co.kr/program-api/1.0/menu/%s' % programcd)

        shareimg = menu['program']['shareimg'].replace('_w640_h360', '_ori')
        if shareimg not in metadata.art:
          try: metadata.art[shareimg] = Proxy.Preview(HTTP.Request(shareimg), sort_order = len(metadata.art) + 1)
          except Exception, e: Log(str(e))

        # http://static.apis.sbs.co.kr/play-api/1.0/sbs_vodalls?...
        vods = JSON.ObjectFromURL('http://static.apis.sbs.co.kr/play-api/1.0/sbs_vodalls?offset=%d&limit=%d&sort=new&search=&cliptype=&subcategory=&programid=%s&absolute_show=Y&mdadiv=01&viewcount=Y' %
            ( 0, 2000, menu['program']['channelid'] + '_V' + menu['program']['programid'][-10:] ), max_size = JSON_MAX_SIZE)
        for v in vods['list']:
          # Log('%s %s-%s %s' % (v['broaddate'], v['content']['contentnumber'], v['content']['cornerid'], v['content']['contenttitle'] ))
          if v['content']['cornerid'] != 0: # 스페셜
            continue
          season_num = '1'
          episode_date = Datetime.ParseDate(v['broaddate'], '%Y-%m-%d').date()   # fix TZ
          episode_nums = []
          match = Regex(u'^\[(\d+)&(\d+)회차 통합본\]').search(v['synopsis'])
          if match:
            episode_nums.append(match.group(1))
            episode_nums.append(match.group(2))
          else:
            episode_nums.append(v['content']['contentnumber'])
          for episode_num in episode_nums:
            date_based_season_num = episode_date.year
            date_based_episode_num = episode_date.strftime('%Y-%m-%d')
            if ((season_num in media.seasons and episode_num in media.seasons[season_num].episodes) or
                (date_based_season_num in media.seasons and date_based_episode_num in media.seasons[date_based_season_num].episodes)):
              episode = metadata.seasons[season_num].episodes[episode_num]
              if episode.summary and u'회차정보가 없습니다' not in episode.summary:
                continue
              episode.summary = String.DecodeHTMLEntities(String.StripTags(v['synopsis'])).strip()
              episode.originally_available_at = episode_date
              episode.title = v['content']['contenttitle'].strip()
              episode.rating = None
      except Exception, e:
        Log.Debug(repr(e))
        pass

    elif 'program.kbs.co.kr' in replay_url:
      try:
        # http://program.kbs.co.kr/2tv/enter/gagcon/pc/list.html?smenu=c2cc5a
        source, sname, stype, smenu = Regex('program.kbs.co.kr/(.+?)/(.+?)/(.+?)/pc/list.html\?smenu=(.+)$').search(replay_url).group(1, 2, 3, 4)

        # http://pprogramapi.kbs.co.kr/api/v1/page?platform=P&smenu=c2cc5a&source=2tv&sname=enter&stype=gagcon&page_type=list
        menu = JSON.ObjectFromURL('http://pprogramapi.kbs.co.kr/api/v1/page?platform=P&smenu=%s&source=%s&sname=%s&stype=%s&page_type=list' %
            ( smenu, source, sname, stype ))

        image_h = menu['data']['site']['meta']['image_h']
        if image_h not in metadata.posters:
          try: metadata.posters[image_h] = Proxy.Preview(HTTP.Request(image_h), sort_order = len(metadata.posters) + 1)
          except Exception, e: Log(str(e))

        image_w = menu['data']['site']['meta']['image_w']
        if image_w not in metadata.art:
          try: metadata.art[image_w] = Proxy.Preview(HTTP.Request(image_w), sort_order = len(metadata.art) + 1)
          except Exception, e: Log(str(e))

        page = 1
        while True:
          # https://ummsapi.kbs.co.kr/landing/contents/episode/list?rtype=jsonp&sort_option=rdatetime%20desc&program_code=T2000-0065&page=1&page_size=9&&callback=angular.callbacks._0
          res = JSON.ObjectFromURL('https://ummsapi.kbs.co.kr/landing/contents/episode/list?rtype=json&sort_option=rdatetime%%20desc&program_code=%s&page=%d&page_size=%d' %
              ( menu['data']['site']['meta']['program_code'], page, 500 ))
          if 'error_msg' in res:
            Log.Debug(res['error_msg'])
            break

          for v in res['data']:
            # Log('%s %s %s' % (v['program_date'], v['program_number'], v['program_subtitle'] or v['description']))
            if not v['program_number']:
              continue
            season_num = '1'
            episode_num = v['program_number']
            episode_date = Datetime.ParseDate(v['program_date'], '%Y%m%d').date()
            date_based_season_num = episode_date.year
            date_based_episode_num = episode_date.strftime('%Y-%m-%d')
            if ((season_num in media.seasons and episode_num in media.seasons[season_num].episodes) or
                (date_based_season_num in media.seasons and date_based_episode_num in media.seasons[date_based_season_num].episodes)):
              episode = metadata.seasons[season_num].episodes[episode_num]
              if episode.summary and u'회차정보가 없습니다' not in episode.summary:
                continue
              episode.summary = v['program_summary']
              episode.originally_available_at = episode_date
              episode.title = v['program_subtitle'] or v['description'] or date_based_episode_num
              episode.rating = None     # float(v['avg_rating'])

          page += 1
          if page > res['page_count']:
            break

      except Exception, e:
        Log.Debug(repr(e))
        pass

    elif 'home.ebs.co.kr' in replay_url:
      try:
        # http://home.ebs.co.kr/bestdoctors/review
        #  => http://home.ebs.co.kr/bestdoctors/replay/1/list;jsessionid=...?courseId=BP0PAPG0000000014&stepId=01BP0PAPG0000000014
        # http://home.ebs.co.kr/baddog/replay/24/list?courseId=10016245&stepId=10035139
        match = Regex('courseId=(.+)&stepId=(.+)').search(replay_url)
        if match:
          courseId, stepId = match.group(1, 2)
          page = 1
          while True:
            html = HTML.ElementFromURL('http://www.ebs.co.kr/tv/show/vodListNew', values={
                'courseId': courseId,
                'stepId': stepId,
                'lectId': '666',    # '10962899',
                'vodStepNm': '',    # '세상에 나쁜 개는 없다 시즌3',
                # 'srchType': '',
                # 'srchText': '',
                # 'srchYear': '',
                # 'srchMonth': '',
                'pageNum': page,
                # 'vodProdId': ''
            }, sleep = 0.5)
            for a in html.xpath('//ul[@class="_playList"]/li//a'):
              season_num = '1'
              episode_date = Datetime.ParseDate(a.xpath('./span[@class="date"]')[0].text, '%Y.%m.%d').date()
              match = Regex(u'^(\d+)회').search(a.text.strip())
              if match:
                episode_num = match.group(1)
              else:
                episode_num = episode_date.strftime('%y%m%d')
              date_based_season_num = episode_date.year
              date_based_episode_num = episode_date.strftime('%Y-%m-%d')
              if ((season_num in media.seasons and episode_num in media.seasons[season_num].episodes) or
                  (date_based_season_num in media.seasons and date_based_episode_num in media.seasons[date_based_season_num].episodes)):
                episode = metadata.seasons[season_num].episodes[episode_num]
                if episode.summary and u'회차정보가 없습니다' not in episode.summary:
                  continue
                # Log('E: S%s E%s %s %s' % (season_num, episode_num, episode_date, a.text.strip()))
                show = HTML.ElementFromURL('http://www.ebs.co.kr/tv/show?prodId=&lectId=%s' % Regex('selVodList\(\'(\d+?)\'').search(a.get('href')).group(1), sleep = 0.5)
                episode.summary = show.xpath('//p[@class="detail_story"]')[0].text.strip()
                episode.originally_available_at = episode_date
                episode.title = a.text.strip() or date_based_episode_num
                episode.rating = None

            page += 1
            if page > min(20, int(''.join(html.xpath('//span[@class="pro_vod_page"]//text()')).strip().split(' / ')[1])):
              break

      except Exception, e:
        Log.Debug(repr(e))
        pass

    else:
      Log(replay_url)

  #   # (5) fill missing info
  #   # if Prefs['override_tv_id'] != 'None':
  #   #   page = HTTP.Request(DAUM_TV_DETAIL2 % metadata.id).content
  #   #   match = Regex('<em class="title_AKA"> *<span class="eng">([^<]*)</span>').search(page)
  #   #   if match:
  #   #     metadata.original_title = match.group(1).strip()

####################################################################################################
class DaumMovieAgent(Agent.Movies):
  name = "Daum Movie"
  languages = [Locale.Language.Korean]
  primary_provider = True
  accepts_from = ['com.plexapp.agents.localmedia']

  def search(self, results, media, lang, manual=False):
    return searchDaumMovie(results, media, lang)

  def update(self, metadata, media, lang):
    Log.Info("in update ID = %s" % metadata.id)
    updateDaumMovie(metadata)

    # override metadata ID
    if Prefs['override_movie_id'] != 'None':
      title = metadata.original_title if metadata.original_title else metadata.title
      if Prefs['override_movie_id'] == 'IMDB':
        url = IMDB_TITLE_SRCH % urllib.quote_plus("%s %d" % (title.encode('utf-8'), metadata.year))
        page = HTTP.Request( url ).content
        match = RE_IMDB_ID.search(page)
        if match:
          metadata.id = match.group(1)
          Log.Info("override with IMDB ID, %s" % metadata.id)

class DaumMovieTvAgent(Agent.TV_Shows):
  name = "Daum Movie"
  primary_provider = True
  languages = [Locale.Language.Korean]
  accepts_from = ['com.plexapp.agents.localmedia']

  def search(self, results, media, lang, manual=False):
    return searchDaumTV(results, media, lang)

  def update(self, metadata, media, lang):
    Log.Info("in update ID = %s" % metadata.id)
    updateDaumTV(metadata, media)

    # override metadata ID
    if Prefs['override_tv_id'] != 'None':
      title = metadata.original_title if metadata.original_title else metadata.title
      if Prefs['override_tv_id'] == 'TVDB':
        url = TVDB_TITLE_SRCH % urllib.quote_plus(title.encode('utf-8'))
        xml = XML.ElementFromURL( url )
        node = xml.xpath('/Data/Series/seriesid')
        if node:
          metadata.id = node[0].text
          Log.Info("override with TVDB ID, %s" % metadata.id)
